#include "IMAGEDATA.h"
#include <rom/tjpgd.h> //TJpgDec (C)ChaN, 2012

IMAGEDATA::IMAGEDATA( )
{
    Width = 0 ;
    Hight = 0 ;
    Image = NULL ;
}

IMAGEDATA::~IMAGEDATA( )
{
    if (Image != NULL) {
        free(Image) ;
    }
}

bool IMAGEDATA::allocImage(int imageSize)
{
    if (Image != NULL) {
        free(Image) ;
    }
    Image = (uint8_t *)malloc(imageSize) ;
    return Image != NULL ;
}

// ▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼
// △▽ JPEG LOAD △▽△▽△▽△▽
UINT in_func(JDEC* jd, BYTE* buff, UINT nbyte)
{
  IODEV *dev = (IODEV*)jd->device;
  UINT ret = 0;
  if (nbyte > dev->ImageDataSize) {
    nbyte = dev->ImageDataSize ;
  }

  if (buff) {
    memcpy(buff,dev->dp,nbyte) ;
    dev->dp += nbyte ;
    dev->ImageDataSize -= nbyte ;
    ret = nbyte;
    return ret ;
  } else {
    // Remove bytes from input stream 
    if(nbyte > 0){ //seek mode:SeekCur=1(FS.h)
      dev->dp += nbyte ;
      dev->ImageDataSize -= nbyte ;
      ret = nbyte;
    }else{
      ret = 0;
    }
  }
  return ret;
}
 
UINT out_func(JDEC* jd, void* bitmap, JRECT* rect)
{
  IODEV *dev = (IODEV*)jd->device;
  uint8_t *src, *dst;
  uint16_t x, y, bwd;
 
  src = (uint8_t*)bitmap;
  //use RGB565
  dst = dev->fbuf + 2 * (rect->top * dev->wfbuf + rect->left);
  
  //bws = 2 * (rect->right - rect->left + 1);
  bwd = 2 * dev->wfbuf;

  uint8_t msb, lsb, tmpR, tmpG , tmpB;
  int16_t xcnt;
  for (y = rect->top; y <= rect->bottom; y++) {
    xcnt = 0;
    for (x = rect->left; x <= rect->right; x++){
      tmpR = *(src++) & 0b11111000;
      tmpG = *(src++) & 0b11111100;
      tmpB = *(src++) & 0b11111000;
      msb = tmpR | (tmpG >> 5);
      lsb = (tmpG << 3) | (tmpB >> 3);
      *(dst + xcnt++) = lsb;
      *(dst + xcnt++) = msb;
    }
    dst += bwd;
  }
 
  return 1;    // Continue to decompress 
}

bool LoadImage(uint8_t *iconData,IMAGEDATA *imageData) {
  void *work;       /* Pointer to the decompressor work area */
  IODEV devid;      /* User defined device identifier */
  JDEC jdec;        /* Decompression object */
  JRESULT res;      /* Result code of TJpgDec API */
  bool result = true ;
  
  if(iconData == NULL){
    return false;
  }

  devid.ImageDataSize = iconData[0] * 256 + iconData[1] ;
  devid.dp = &iconData[6] ;
  devid.fbuf = NULL ;
  devid.wfbuf = 0 ;

  // Allocate a work area for TJpgDec 
  const int WorkBuffSize = 1024*4 ;
  work = malloc(WorkBuffSize);
  
  // Prepare to decompress 
  res = jd_prepare(&jdec, in_func, work, WorkBuffSize, &devid);
  if (res == JDR_OK) {
    // Ready to dcompress. Image info is available here. 
    imageData->setWidth(jdec.width) ;
    imageData->setHight(jdec.height) ;
    imageData->allocImage(2 * jdec.width * jdec.height) ; //RGB565
    devid.fbuf = (BYTE *)imageData->getImage( ) ;
    if (devid.fbuf != NULL) {
      devid.wfbuf = jdec.width;
    }
    res = jd_decomp(&jdec, out_func, 0);   // Start to decompress with 1/1 scaling 
    if (res == JDR_OK) {
      // Decompression succeeded. You have the decompressed image in the frame buffer here. 
    } else {
    }
  } else {
    result = false ;
  }
  
  free(work);             // Discard work area 
  work = NULL ;

  if (!result) {
    return result ;
  }

  if(devid.fbuf){
    devid.fbuf = NULL;
  }
  return result;
}
